/* FORMANT3 - sample code for a basic filter (3 band-passes [formants]) instrument

   p0 = output start time
   p1 = input start time
   p2 = input duration
   p3 = amplitude multiplier
   p4 = filter1 center frequency
   p5 = filter1 bandwidth
   p6 = filter2 center frequency
   p7 = filter2 bandwidth
   p8 = filter3 center frequency
   p9 = filter3 bandwidth
   p10 = input channel [optional, default is 0]
   p11 = pan (in percent-to-left format) [optional, default is 0]
*/

#include <stdio.h>
#include <stdlib.h>
#include <ugens.h>
#include <Ougens.h>
#include "FORMANT3.h"          // declarations for this instrument class
#include <rt.h>
#include <rtdefs.h>


FORMANT3::FORMANT3()
	: resetter(0)
{
}

FORMANT3::~FORMANT3()
{
}


int FORMANT3::init(double p[], int n_args)
{
    if (rtsetoutput(p[0], p[2], this) == -1)
		return DONT_SCHEDULE;

	if (outputChannels() > 2)
		return die("FORMANT3", "Use mono or stereo output only.");

	if (rtsetinput(p[1], this) == -1)
		return DONT_SCHEDULE;

    inchan = p[10];
	if (inchan >= inputChannels())
		return die("FORMANT3", "You asked for channel %d of a %d-channel input.",
		                                             inchan, inputChannels());

    // set up our input object
    theInput = new Ortgetin(this);

    // set up our filters
    // Oreson(float SR, float centerFreq, float bandwidth[, Scale scaling])
    theFilt1 = new Oreson(SR, p[4], p[5]);
    theFilt2 = new Oreson(SR, p[6], p[7]);
    theFilt3 = new Oreson(SR, p[8], p[9]);
    
    
    amp = p[3];
    pan = p[11];

	return nSamps();
}


int FORMANT3::configure()
{
    return 0;	// IMPORTANT: Return 0 on success, and -1 on failure.
}


void FORMANT3::doupdate()
{
	double p[8];
	update(p, 8);

	amp = p[3];
    pan = p[7];
}


int FORMANT3::run()
{
    float out[2];
    float in[2];

	// Each loop iteration processes 1 sample frame.
	for (int i = 0; i < framesToRun(); i++) {
		if (--resetter <= 0) {
			doupdate();
			resetter = getSkip();
		}

        // Grab the current input sample, scaled by the amplitude multiplier.
        theInput->next(in);

        // filter it!
        in[inchan] *= amp;
		out[0] = theFilt1->next(in[inchan]);
		out[0] += theFilt2->next(in[inchan]);
		out[0] += theFilt3->next(in[inchan]);


		// If we have stereo output, use the pan pfield.
		if (outputChannels() == 2) {
			out[1] = out[0] * (1.0 - pan);
			out[0] *= pan;
		}

		// Write this sample frame to the output buffer.
		rtaddout(out);

		// Increment the count of sample frames this instrument has written.
		increment();
	}

	// Return the number of frames we processed.
	return framesToRun();
}


Instrument *makeFORMANT3()
{
	FORMANT3 *inst = new FORMANT3();
	inst->set_bus_config("FORMANT3");

	return inst;
}


void rtprofile()
{
	RT_INTRO("FORMANT3", makeFORMANT3);
}


